////////////////////////////////////////////////////////////////
//
// Copyright (C) 2006 Affymetrix, Inc.
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License 
// (version 2.1) as published by the Free Software Foundation.
// 
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
// for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
//
////////////////////////////////////////////////////////////////


/**
 * @file   tsv-join-test.cpp
 * @brief  Program for doing regression tests on tsv-join.
 */
#include <vector>
#include <string>
#include "util/RegressionTest.h"
#include "util/TextFileCheck.h"
#include "util/Util.h"
#include "util/Verbose.h"

using namespace std;

class TsvJoinTest
{

public:
  int numPassed, numFailed;
  TsvJoinTest()
  {
    numPassed = 0;
    numFailed = 0;
  }
  void doTranscriptClusterTest();
  void doProbesetIdTest();

private:
  string errorMsg;
};

void TsvJoinTest::doTranscriptClusterTest()
{
  // apt-tsv-join does not make an output directory
  Util::createDir ("tsv-join-test-generated");
  string command = "./apt-tsv-join -k transcript_cluster_id -o tsv-join-test-generated/join-transcript-cluster.txt data/HuEx-1_0-st-probeset-annot.85000.csv  data/HuEx-1_0-st-transcript-annot.16000.csv";
  vector<RegressionCheck *> checks;
  RegressionTest joinTest (command.c_str(), checks);
  Verbose::out (1, "Doing doTranscriptClusterTest()");
  if (!joinTest.run())
  {
    Verbose::out (1, "Error in TsvJoinTest::doTranscriptClusterTest(): " + joinTest.getErrorMsg());
    numFailed++;
  }
  else
  {
    TextFileCheck textCheck ("tsv-join-test-generated/join-transcript-cluster.txt",
                             "data/join-transcript-cluster.txt", 34);
    if (textCheck.check (errorMsg))
    {
      numPassed++;
      Verbose::progressStep (1);
    }
    else
    {
      Verbose::out (1, "Error in TsvJoinTest::doTranscriptClusterTest(): " + errorMsg);
      numFailed++;
    }
  }
}

void TsvJoinTest::doProbesetIdTest()
{
  Util::createDir ("tsv-join-test-generated");
  string command = "./apt-tsv-join -k probeset_id  -o tsv-join-test-generated/join-probeset-id.txt data/HuEx-1_0-st-transcript-annot.csv data/quant-norm.pm-gcbg.iter-plier.summary.txt";
  vector<RegressionCheck *> checks;
  RegressionTest joinTest (command.c_str(), checks);
  Verbose::out (1, "Doing doProbesetIdTest()");
  if (!joinTest.run())
  {
    Verbose::out (1, "Error in TsvJoinTest::doProbesetIdTest(): " + joinTest.getErrorMsg());
    ++numFailed;
  }
  else
  {
    TextFileCheck textCheck ("tsv-join-test-generated/join-probeset-id.txt",
                             "data/join-probeset-id.txt", 34);
    if (textCheck.check (errorMsg))
    {
      numPassed++;
      Verbose::progressStep (1);
    }
    else
    {
      Verbose::out (1, "Error in TsvJoinTest::doTranscriptClusterTest(): " + errorMsg);
      numFailed++;
    }
  }
}

int main (int argc, char* argv[])
{
  TsvJoinTest test;
  Verbose::setLevel(2);
  Verbose::progressBegin (1, "Doing tests", 20, 1, 20);
  test.doTranscriptClusterTest();
  test.doProbesetIdTest();
  Verbose::progressEnd (1, "Done.");
  Verbose::out (1, "NumPassed: " + ToStr (test.numPassed) + " NumFailed: " + ToStr (test.numFailed));
  return test.numFailed != 0;
}
