/*
 *  This file is part of BGX, the Bayesian Gene eXpression program.
 *  Copyright (c) 2008 Ernest Turro <ernest.turro@ic.ac.uk>
 *
 *  BGX is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  BGX is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <algorithm>
#include <map>
#include <ctype.h>
#include <cstdlib>
#include <cstring>
#ifdef WIN32
 #include <io.h>
#else
  #include <sys/stat.h>
#endif
#include <dirent.h>
#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
 
using namespace std;

extern "C" {

void writePMs(int *intensities, int* noOfIntensities, int* numArrays, char **inputIndicesFile /* 1-indexed ! */, char **outputFile, int *noOfProbes) {

  // Read in variable length PMinds (identical probes on one line)
  vector<vector<int> > PMinds;
  ifstream ifs(*inputIndicesFile); 
  char str[16384];
  *noOfProbes=0; 
  while(1){
    vector<int> curInds;
    ifs.getline(str, 16384);
    if(ifs.eof()) break;
    (*noOfProbes)++;
    char buf[16];
    int i=0;
    while(1) {
      int j=0;
      while(isdigit(str[i])) {
        buf[j] = str[i];
        j++;
        i++;
      }
      buf[j] = '\0';
      curInds.push_back(atoi(buf)-1);
      while(!isdigit(str[i]) && str[i] != '\0') i++;
      if(str[i] == '\0') break;
    }
    PMinds.push_back(curInds);
  }
  ifs.close();

  // Output average intensity for identical probes for each array
  vector<int> currentIndices;
  int currentSumIntensities;
  int noOfIdenticalProbes;
  ofstream ofs(*outputFile);
  for(int a=0; a < *numArrays ; a++) {
    vector<vector <int> >::iterator it;
    for(it = PMinds.begin(); it != PMinds.end(); it++) {
      currentSumIntensities=noOfIdenticalProbes=0;
      vector<int>::iterator it2;
      for(it2= it->begin(); it2 != it->end(); it2++) {
        currentSumIntensities += intensities[*noOfIntensities*a + *it2];
        noOfIdenticalProbes++;
      }
      ofs << ((double)currentSumIntensities/noOfIdenticalProbes) << endl;
    }
  }
  ofs.close();

}
}
/*
int main(int argc, char **argv) {

  ifstream ifs(argv[1]); //intensities
  int *intensities = new int[7000000];
  int i=0;
  while(!ifs.eof()) {
    ifs >> intensities[i];
    i++;
  }
  ifs.close();
  writePMids(intensities, &argv[2], &argv[3]); 

  delete intensities;
  return 0;
}
*/

